@@ -1,21 +1,22 @@
 package Test::Script;
 
 # ABSTRACT: Basic cross-platform tests for scripts
-our $VERSION = '1.19_03'; # TRIAL VERSION
+our $VERSION = '1.19_04'; # TRIAL VERSION
 
 
-use 5.006;
+use 5.008001;
 use strict;
 use warnings;
-use Carp             ();
-use Exporter         ();
-use File::Spec       ();
-use File::Spec::Unix ();
-use Probe::Perl      ();
-use Capture::Tiny    qw( capture );
-use Test::Builder    ();
-use File::Temp       ();
-use File::Path       ();
+use Carp qw( croak );
+use Exporter;
+use File::Spec;
+use File::Spec::Unix;
+use Probe::Perl;
+use Capture::Tiny qw( capture );
+use Test::Builder ();
+use File::Temp qw( tempdir );
+use File::Path qw( rmtree );
+use IO::Handle;
 
 our @ISA     = 'Exporter';
 our @EXPORT  = qw{
@@ -56,10 +57,10 @@ sub perl () {
 sub path ($) {
   my $path = shift;
   unless ( defined $path ) {
-    Carp::croak("Did not provide a script name");
+    croak("Did not provide a script name");
   }
   if ( File::Spec::Unix->file_name_is_absolute($path) ) {
-    Carp::croak("Script name must be relative");
+    croak("Script name must be relative");
   }
   File::Spec->catfile(
     File::Spec->curdir,
@@ -87,7 +88,7 @@ sub script_compiles {
     $error eq '' and $exit == 0 and $signal == 0 and $stderr =~ /syntax OK\s+\z/si
   );
 
-  File::Path::rmtree($dir);
+  rmtree($dir);
 
   my $test = Test::Builder->new;
   $test->ok( $ok, $_[0] || "Script $unix compiles" );
@@ -104,7 +105,7 @@ sub script_compiles {
 # safer as very long argument lists can break calls to system
 sub _preload_module
 {
-  my $dir = File::Temp::tempdir( CLEANUP => 1 );
+  my $dir = tempdir( CLEANUP => 1 );
   # this is hopefully a pm file that nobody would use
   my $filename = File::Spec->catfile($dir, '__TEST_SCRIPT__.pm');
   my $fh;
@@ -135,13 +136,45 @@ sub script_runs {
   my $cmd    = [ perl, @$pargs, "-I$dir", '-M__TEST_SCRIPT__', $path, @$args ];
   $stdout = '';
   $stderr = '';
+
+  if($opt->{stdin})
+  {
+    my $filename;
+
+    if(ref($opt->{stdin}) eq 'SCALAR')
+    {
+      $DB::single = 1;
+      $filename = File::Spec->catfile(
+        tempdir(CLEANUP => 1),
+        'stdin.txt',
+      );
+      my $tmp;
+      open($tmp, '>', $filename) || die "unable to write to $filename";
+      print $tmp ${ $opt->{stdin} };
+      close $tmp;
+    }
+    elsif(ref($opt->{stdin}) eq '')
+    {
+      $filename = $opt->{stdin};
+    }
+    else
+    {
+      croak("stdin MUST be either a scalar reference or a string filename");
+    }
+
+    my $fh;
+    open($fh, '<', $filename) || die "unable to open $filename $!";
+    STDIN->fdopen( $fh, 'r' ) or die "unable to reopen stdin to $filename $!";
+  }
+
   (${$opt->{stdout}}, ${$opt->{stderr}}) = capture { system(@$cmd) };
+  
   my $error  = $@;
   my $exit   = $? ? ($? >> 8) : 0;
   my $signal = $? ? ($? & 127) : 0;
   my $ok     = !! ( $error eq '' and $exit == $opt->{exit} and $signal == $opt->{signal} );
 
-  File::Path::rmtree($dir);
+  rmtree($dir);
 
   my $test = Test::Builder->new;
   $test->ok( $ok, $_[0] || "Script $unix runs" );
@@ -254,7 +287,7 @@ sub _script {
       return [ @$in ];
     }
   }
-  Carp::croak("Invalid command parameter");
+  croak("Invalid command parameter");
 }
 
 # Determine any extra arguments that need to be passed into Perl.
@@ -279,7 +312,7 @@ sub _options {
   $options{exit}   = 0        unless defined $options{exit};
   $options{signal} = 0        unless defined $options{signal};
   my $stdin = '';
-  $options{stdin}  = \$stdin  unless defined $options{stdin};
+  #$options{stdin}  = \$stdin  unless defined $options{stdin};
   $options{stdout} = \$stdout unless defined $options{stdout};
   $options{stderr} = \$stderr unless defined $options{stderr};
 
@@ -313,7 +346,7 @@ Test::Script - Basic cross-platform tests for scripts
 
 =head1 VERSION
 
-version 1.19_03
+version 1.19_04
 
 =head1 SYNOPSIS
 
@@ -410,7 +443,7 @@ scalar.
 =back
 
 The behavior for any other types is undefined (the current implementation uses
-L<Capture::Tiny>).
+L<Capture::Tiny>).  Any already opened stdin will be closed.
 
 =item stdout
 
@@ -497,10 +530,13 @@ expression.
 
 =head1 CAVEATS
 
-This module is fully supported back to Perl 5.8.1.  It may work on 5.8.0.
-It should work on Perl 5.6.x and I may even test on 5.6.2.  I will accept
-patches to maintain compatibility for such older Perls, but you may
-need to fix it on 5.6.x / 5.8.0 and send me a patch.
+This module is fully supported back to Perl 5.8.1.  In the near future, support
+for the older pre-Test2 Test::Builer will be dropped.
+
+The STDIN handle will be closed when using script_runs with the stdin option.
+An older version used L<IPC::Run3>, which attempted to save STDIN, but
+apparently this cannot be done consistently or portably.  We now use
+L<Capture::Tiny> instead and explicitly do not support saving STDIN handles.
 
 =head1 SEE ALSO
 
